import Foundation

/**
Oberklasse aller Bausteine des Labyrinths

- author: Albert Wiedemann
- version: 1.0
 */
class LabyrinthElement
{
    /** Die Seitenlänge elens Elementquadrats */
    static let seitenlänge = 20
    /** Das Darstellungsrechteck */
    fileprivate let darstellung: Rechteck
    /** x-Position des Quadrats in Zellenangaben */
    fileprivate var x: Int
    /** y-Position des Quadrats in Zellenangaben */
    fileprivate var y: Int

    /**
	Legt das Darstellungsrechteck an.
	- parameters:
		- x: x-Position des Quadrats in Zellenangaben
		- y: y-Position des Quadrats in Zellenangaben
     */
    init(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        darstellung = Rechteck()
        darstellung.PositionSetzen(x: x * LabyrinthElement.seitenlänge, y: y * LabyrinthElement.seitenlänge);
        darstellung.GrößeSetzen(breite: LabyrinthElement.seitenlänge, höhe: LabyrinthElement.seitenlänge);
    }
    
    /**
	Gibt die x-Position des Elements zurück.
	- returns: x-Position
    */
    func XPositionGeben() -> Int
    {
        return x
    }
    
    /**
	Gibt die y-Position des Elements zurück.
	- returns: y-Position
     */
    func YPositionGeben() -> Int
    {
        return y
    }
    
    /**
	Setzt die Sichtbarkeit des Elements
	- parameters:
		- sichtbar: wenn wahr, ist das Element sichtbar
    */
    func SichtbarkeitSetzen(sichtbar: Bool)
    {
        darstellung.SichtbarkeitSetzen(sichtbar: sichtbar)
    }
}

/**
Ein Gangbaustein des Labyrinths

- author: Albert Wiedemann
- version: 1.0
*/
class Gang: LabyrinthElement
{

    /**
	Legt das Darstellungsrechteck in der Gangfarbe an.
	- parameters:
		- x: x-Position des Quadrats in Zellenangben
		- y: y-Position des Quadrats in Zellenangben
     */
    override init(x: Int, y: Int)
    {
        super.init(x: x, y: y)
        darstellung.FarbeSetzen(farbe: "weiß")
    }
}

/**
Ein Ausgangbaustein des Labyrinths

- author: Albert Wiedemann
- version: 1.0
*/
class Ausgang: Gang
{

    /**
	Legt das Darstellungsrechteck in der Ausgangfarbe an.
	- parameters:
		- x: x-Position des Quadrats in Zellenangben
		- y: y-Position des Quadrats in Zellenangben
     */
    override init(x: Int, y: Int)
    {
        super.init(x: x, y: y)
        darstellung.FarbeSetzen(farbe: "grün")
    }
}

/**
Ein Mauerbaustein des Labyrinths

- author: Albert Wiedemann
- version: 1.0
*/
class Mauer: LabyrinthElement
{

    /**
	Legt das Darstellungsrechteck in der Mauerfarbe an.
	- parameters:
		- x: x-Position des Quadrats in Zellenangben
		- y: y-Position des Quadrats in Zellenangben
     */
    override init(x: Int, y: Int)
    {
        super.init(x: x, y: y)
        darstellung.FarbeSetzen(farbe: "schwarz")
    }
}

/**
Theseus erhält einen Weg, auf dem er zum Ausgang findet

- author: Albert Wiedemann
- version: 1.0
*/
class Theseus: Figur
{
    /** Der Weg zum Ausgang */
    private var weg: [Knoten]
    /** Bereits zurückgelegt Schritte auf dem Weg zum Ausgang */
    private var schritte: Int
    /** x-Koordinate des Wanderers in Labyrinthelementen */
    private var x: Int
    /** y-Koordinate des Wanderers in Labyrinthelementen */
    private var y: Int
    
    /**
	Legt die Figur und die Startposition fest.
    */
    init(x: Int, y: Int)
    {
        weg = [Knoten]()
        schritte = 0
        self.x = x
        self.y = y
        super.init()
        FigurFestlegen()
        GrößeSetzen(größe: LabyrinthElement.seitenlänge);
        PositionSetzen(x: x * LabyrinthElement.seitenlänge + LabyrinthElement.seitenlänge / 2, y: y  * LabyrinthElement.seitenlänge + LabyrinthElement.seitenlänge / 2)
    }
    
    /**
	Legt das Aussehen von Theseus fest.
     */
    private func FigurFestlegen()
    {
        EigeneFigurLöschen();
        FigurteilFestlegenRechteck(x: -35, y: 0, breite: 70, höhe: 25, farbe: "rot")
        FigurteilFestlegenEllipse(x: -25, y: 25, breite: 10, höhe: 10, farbe: "schwarz")
        FigurteilFestlegenEllipse(x: 15, y: 25, breite: 10, höhe: 10, farbe: "schwarz")
        FigurteilFestlegenEllipse(x: -20, y: -40, breite: 40, höhe: 40, farbe: "rot")
        FigurteilFestlegenEllipse(x: 5, y: -27, breite: 15, höhe: 15, farbe: "gelb")
    }
    
    /**
	Legt den Weg um Ausgang fest.
	- parameters:
		- weg: der Weg zum Ausgang
    */
    func WegSetzen (weg: [Knoten])
    {
        self.weg = weg
        schritte = 1
    }
    
    /**
	Bewegt Theseus ein Feld in Richtung Ausgang
    */
    override func AktionAusführen()
    {
        if schritte < weg.count
        {
            let k = weg[schritte]
            if k.XGeben() == x + 1
            {
                WinkelSetzen(winkel: 0)
            }
            else if k.XGeben() == x - 1
            {
                WinkelSetzen(winkel: 180)
            }
            else if k.YGeben() == y - 1
            {
                WinkelSetzen(winkel: 90)
            }
            else if k.YGeben() == y + 1
            {
                WinkelSetzen(winkel: 270)
            }
            Gehen(länge: Double(LabyrinthElement.seitenlänge));
            x = k.XGeben()
            y = k.YGeben()
            schritte += 1
        }
    }
}

/**
Verwaltet das Labyrinth

- author: Albert Wiedemann
- version: 1.0
*/
class Labyrinth
{
    /** Feld der Labyrinthelemente */
    private var elemente: [[LabyrinthElement?]]
    /** Zufallsgenrator */
    private let zufall: Zufall
    /** x-Koordinate des Ausgangs */
    private var xAusgang: Int
    /** y-Koordinate des Ausgangs */
    private var yAusgang: Int
    /** x-Koordinate des Startpunkts */
    private var xStart: Int
    /** y-Koordinate des Startpunkts */
    private var yStart: Int

    /**
	Legt die Attribute an.
	- parameters:
		- breite: Breite des Labyrinths in Zellenanzahl
		- höhe: Höhe des Labyrinths in Zellenanzahl
    */
    init(breite: Int, höhe: Int)
    {
        zufall = Zufall()
        elemente = [[LabyrinthElement?]]()
        for b in 0 ..< breite
        {
            elemente.append([LabyrinthElement?]())
            for _ in 0 ..< höhe
            {
                elemente[b].append(nil)
            }
        }
        xAusgang = -1
        yAusgang = -1
        xStart = -1
        yStart = -1
    }
    
    /**
	Generiert das Labyrinth
    */
    func Generieren()
    {
        var dx: Int
        var dy: Int
        var xmax = elemente.count
        var ymax = elemente[0].count
        var minSchritte = xmax * ymax / 3
        var schritte = 1
        var x = zufall.ZufallGanz(top: xmax - 2) + 1
        var y = zufall.ZufallGanz(top: ymax - 2) + 1
        elemente[x][y] = Gang (x: x, y: y)
        
        while(true)
        {
            repeat
            {
                dx = zufall.ZufallGanz(top: 3) - 1
                dy = zufall.ZufallGanz(top: 3) - 1
            } while abs(dx) == abs(dy)
            x += dx
            y += dy
            if (x == 0) || (y == 0) || (x == xmax - 1) || (y == ymax - 1)
            {
                if (schritte > minSchritte) && (elemente[x][y] == nil)
                {
                    elemente[x][y] = Ausgang(x: x, y: y)
                    xAusgang = x
                    yAusgang = y
                    break
                }
                else
                {
                    x -= dx
                    y -= dy
                }
            }
            else if elemente[x][y] == nil
            {
                if (elemente[x - 1][y - 1] != nil) && (elemente[x - 1][y] != nil) && (elemente[x][y - 1] != nil) ||
                   (elemente[x + 1][y - 1] != nil) && (elemente[x + 1][y] != nil) && (elemente[x][y - 1] != nil) ||
                   (elemente[x + 1][y + 1] != nil) && (elemente[x + 1][y] != nil) && (elemente[x][y + 1] != nil) ||
                   (elemente[x - 1][y + 1] != nil) && (elemente[x - 1][y] != nil) && (elemente[x][y + 1] != nil)
                {
                    x -= dx
                    y -= dy
                }
                else
                {
                    schritte += 1
                    elemente[x][y] = Gang(x: x, y: y)
                }
            }
        }
        for b in 0 ..< elemente.count
        {
            for h in 0 ..< elemente[b].count
            {
                if elemente[b][h] == nil
                {
                    elemente[b][h] = Mauer(x: b, y: h)
                }
            }
        }
    }

    /**
	Erzeugt die Graphendarstellung
	- parameters:
		- graph: das Objekt zur Verwaltung des Graphen
    */
    func GraphErzeugen(graph: GraphMatrix)
    {
        for x in 0 ..< elemente.count
        {
            for y in 0 ..< elemente[x].count
            {
                if !(elemente[x][y] is Mauer)
                {
                    graph.KnotenEinfügen(x: x, y: y)
                    if elemente[x][y] is Ausgang
                    {
                        graph.KnotenGeben(x: x, y: y)?.FarbeSetzen(f: "grün")
                    }
                    if (x > 0) && !(elemente[x - 1][y] is Mauer)
                    {
                        graph.KanteEinfügen(x1: x, y1: y, x2: x - 1, y2: y)
                    }
                    if (y > 0) && !(elemente[x][y - 1] is Mauer)
                    {
                        graph.KanteEinfügen(x1: x, y1: y, x2: x, y2: y - 1)
                    }
                }
            }
        }
    }
    
    /**
	Setzt die Sichtbarkeit des Labyrinths
	- parameters:
		- sichtbar: wenn wahr, ist das Labyrinth sichtbar
     */
    func SichtbarkeitSetzen(sichtbar: Bool)
    {
        for x in 0 ..< elemente.count
        {
            for y in 0 ..< elemente[x].count
            {
                elemente[x][y]?.SichtbarkeitSetzen(sichtbar: sichtbar)
            }
        }
    }
    
    /**
	Meldet die x-Koordinate des Ausgangs zurück
	- returns: x-Koordinate des Ausgangs
    */
    func XAusgangGeben() -> Int
    {
        return xAusgang
    }
    
    /**
	Meldet die y-Koordinate des Ausgangs zurück
	- returns: y-Koordinate des Ausgangs
    */
    func YAusgangGeben() -> Int
    {
        return yAusgang
    }
    
    /**
	Startpunkt ermitteln
    */
    func StartpunktGenerieren()
    {
        var x1, y1, x2, y2, x3, y3: Int
        repeat
        {
            x1 = zufall.ZufallGanz(top: elemente.count - 2) + 1
            y1 = zufall.ZufallGanz(top: elemente[0].count - 2) + 1
        } while elemente[x1][y1] is Mauer
        repeat
        {
            x2 = zufall.ZufallGanz(top: elemente.count - 2) + 1
            y2 = zufall.ZufallGanz(top: elemente[0].count - 2) + 1
        } while elemente[x2][y2] is Mauer
        repeat
        {
            x3 = zufall.ZufallGanz(top: elemente.count - 2) + 1
            y3 = zufall.ZufallGanz(top: elemente[0].count - 2) + 1
        } while elemente[x3][y3] is Mauer
        if abs(x1 - xAusgang) + abs(y1 - yAusgang) > abs(x2 - xAusgang) + abs(y2 - yAusgang)
        {
            if abs(x1 - xAusgang) + abs(y1 - yAusgang) > abs(x3 - xAusgang) + abs(y3 - yAusgang)
            {
                xStart = x1
                yStart = y1
            }
            else if abs(x2 - xAusgang) + abs(y2 - yAusgang) > abs(x3 - xAusgang) + abs(y3 - yAusgang)
            {
                xStart = x2
                yStart = y2
            }
            else
            {
                xStart = x3
                yStart = y3
                
            }
        }
        else if abs(x2 - xAusgang) + abs(y2 - yAusgang) > abs(x3 - xAusgang) + abs(y3 - yAusgang)
        {
            xStart = x2
            yStart = y2
        }
        else
        {
            xStart = x3
            yStart = y3
            
        }
    }
    
    /**
	Meldet die x-Koordinate des Ausgangs zurück
	- returns: x-Koordinate des Ausgangs
    */
    func XStartGeben() -> Int
    {
        return xStart
    }
    
    /**
	Meldet die y-Koordinate des Ausgangs zurück
	- returns: y-Koordinate des Ausgangs
    */
    func YStartGeben() -> Int
    {
        return yStart
    }
}

/**
Verwaltet ein Knotensymbol in einem Graph

- author: Albert Wiedemann
- version: 1.0
*/
class KnotenSymbol
{
    /** Außenkreis des Darstellungssymbols */
    private let außen: Kreis
    /** Innenkreis des Darstellungssymbols */
    private let innen: Kreis
    /** Text des Darstellungssymbols */
    private let text: Text
    /** Der Bezeichner */
    private var bezeichner: String
    /** Radius */
    private var r: Int
    /** x-Koordinate */
    private var x: Int
    /** y-Koordinate */
    private var y: Int
    /** Farbe */
    private var farbe: String
    
	/**
	Legt das Symbol an und besetzt die Attribute.
	- parameters:
		- x: x-Koordinate
		- y: y-Koordinate
		- radius: Radius
		- farbe: Farbe
		- bezeichner: Bezeichner
    */
    init(x: Int, y: Int, radius: Int, farbe: String, bezeichner: String)
    {
        self.x = x
        self.y = y
        self.r = radius
        self.farbe = farbe
        self.bezeichner = bezeichner
        außen = Kreis()
        außen.FarbeSetzen(farbe: "schwarz")
        innen = Kreis()
        text = Text()
        text.TextGrößeSetzen(größe: 18)
        DarstellungAktualisieren()
    }
    
    /**
	Aktualisiert das Symbol
    */
    private func DarstellungAktualisieren()
    {
        außen.PositionSetzen(x: x, y: y)
        außen.RadiusSetzen(radius: r)
        innen.PositionSetzen(x: x, y: y)
        innen.RadiusSetzen(radius: r - 2)
        innen.FarbeSetzen(farbe: farbe)
        text.PositionSetzen(x: x - bezeichner.count / 2 * 10 - bezeichner.count % 2 * 5, y: y + 6)
        text.TextSetzen(text: bezeichner)
    }
    
    /**
	Setzt die Farbe der Darstellung
	- parameters:
		- f: die (neue) Farbe
    */
    func FarbeSetzen(f: String)
    {
        farbe = f
        DarstellungAktualisieren()
    }
    
    /**
	Setzt die Position der Darstellung
	- parameters:
		- x: x-Koordinate
		- y: y-Koordinate
    */
    func PositionSetzen(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        DarstellungAktualisieren()
    }
    
    /**
	Setzt den Bezeichner des Symbols
	- parameters:
		- bezeichner: der (neue) Bezeichner
    */
    func BezeichnerSetzen(bezeichner: String)
    {
        self.bezeichner = bezeichner
        DarstellungAktualisieren()
    }
    
    /**
	Meldet die x-Koordinate des Symbols zurück.
	- returns: x-Koordinate
    */
    func XGeben() -> Int
    {
        return x
    }
    
    /**
	Meldet die y-Koordinate des Symbols zurück.
	- returns: y-Koordinate
    */
    func YGeben() -> Int
    {
        return y
    }
    
    /**
	Meldet den Bezeichner des Symbols zurück.
	- returns: Bezeichner
     */
    func BezeichnerGeben() -> String
    {
        return bezeichner
    }
    
    /**
	Entfernt das Knotensymbol aus der Anzeige
    */
    func Entfernen()
    {
        außen.Entfernen()
        innen.Entfernen()
        text.Entfernen()
    }
    
    /**
	Setzt die Sichtbarkeit des Symbols
	- parameters:
		- sichtbar: wenn wahr, ist das Symbol sichtbar
    */
    func SichtbarkeitSetzen(sichtbar: Bool)
    {
        außen.SichtbarkeitSetzen(sichtbar: sichtbar)
        innen.SichtbarkeitSetzen(sichtbar: sichtbar)
        text.SichtbarkeitSetzen(sichtbar: sichtbar)
    }
}

/**
Verwaltet ein Kantensymbol in einem Graph

- author: Albert Wiedemann
- version: 1.0
*/
class KantenSymbol
{
    /** Außenrechteck des Darstellungssymbols */
    private let außen: Rechteck
    /** Innenrechteck des Darstellungssymbols */
    private let innen: Rechteck
    /** Richtungspfeil des Darstellungssymbols */
    private let pfeil: Dreieck
    /** Gewichtsangabe des Darstellungssymbols */
    private let text: Text
    /** Breite des Darstellungssymbols */
    private let breite: Int
    /** Gerichtete Kante */
    private let gerichtet: Bool
    /** Gewicht der Kante */
    private let gewicht: String
    /** Farbe */
    private var farbe: String
    /** Startknoten */
    private let von: KnotenSymbol
    /** Endknoten */
    private let nach: KnotenSymbol

    /**
	Legt das Symbol an und besetzt die Attribute.
	- parameters:
		- start: Startknoten
		- ende: Endknoten
		- gerichtet: wenn wahr, ist der Weg gerichtet
		- gewicht: Kantengewicht
		- breite: Breite der Linie
		- farbe: Farbe der Linie
    */
    init(start: KnotenSymbol, ende: KnotenSymbol, gerichtet: Bool, gewicht: String, breite: Int, farbe: String)
    {
        von = start
        nach = ende;
        self.farbe = farbe
        self.breite = breite
        self.gerichtet = gerichtet
        self.gewicht = gewicht
        außen = Rechteck()
        innen = Rechteck()
        pfeil = Dreieck()
        pfeil.FarbeSetzen(farbe: "schwarz")
        text = Text()
        text.SichtbarkeitSetzen(sichtbar: gewicht != "")
        text.TextSetzen(text: gewicht)
        DarstellungAktualisieren()
    }
    
    /**
    Aktualisiert das Symbol
    */
    func DarstellungAktualisieren()
    {
        var x1 = Double(von.XGeben())
        var y1 = Double(von.YGeben())
        var x2 = Double(nach.XGeben())
        var y2 = Double(nach.YGeben())
        let länge = Int(sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)))
        let xm = Int((x1 + x2) / 2.0)
        let ym = Int((y1 + y2) / 2.0)
        let x = xm - länge / 2
        let y = ym - breite / 2
        außen.PositionSetzen(x: x, y: y)
        außen.GrößeSetzen(breite: länge, höhe: breite)
        if breite < 4
        {
            let größe = breite * 4;
            innen.SichtbarkeitSetzen(sichtbar: false)
            außen.FarbeSetzen(farbe: farbe)
            pfeil.SichtbarkeitSetzen(sichtbar: gerichtet)
            pfeil.PositionSetzen(x: xm, y: ym - größe / 2)
            pfeil.GrößeSetzen(breite: größe, höhe: größe)
            pfeil.WinkelSetzen(winkel: 270)
        }
        else
        {
            let delta = breite <= 8 ? 2 : 4
            innen.SichtbarkeitSetzen(sichtbar: true)
            innen.PositionSetzen(x: x, y: y + delta / 2)
            innen.GrößeSetzen(breite: länge, höhe: breite - delta)
            innen.FarbeSetzen(farbe: farbe)
            außen.FarbeSetzen(farbe: "schwarz")
            pfeil.SichtbarkeitSetzen(sichtbar: gerichtet)
            pfeil.PositionSetzen(x: xm, y: ym - breite / 2)
            pfeil.GrößeSetzen(breite: breite - delta, höhe: breite)
            pfeil.WinkelSetzen(winkel: 270)
        }
        
        var winkel = 0.0
        if x1 == x2
        {
            if y2 > y1
            {
                winkel = -90
            }
            else
            {
                winkel = 90
            }
        }
        else if x1 < x2
        {
            winkel = -atan((y2 - y1) / (x2 - x1)) / Double.pi * 180.0
        }
        else
        {
            winkel = 180 - atan((y2 - y1) / (x2 - x1)) / Double.pi * 180.0
        }
        pfeil.WinkelSetzen(winkel: Int(winkel + 270))
        innen.WinkelSetzen(winkel: Int(winkel))
        außen.WinkelSetzen(winkel: Int(winkel))
        pfeil.GanzNachHintenBringen()
        innen.GanzNachHintenBringen()
        außen.GanzNachHintenBringen()
        let delta = Double(breite)
        let w = (winkel + 90) / 180.0 * Double.pi
        if (-90 <= winkel) && (winkel <= 0)
        {
            text.PositionSetzen(x: xm + Int(delta * cos(w)), y: ym - Int(delta * sin(w)))
        }
        else if (0 < winkel) && (winkel <= 90)
        {
            text.PositionSetzen(x: xm - Int(delta * cos(w)), y: ym + Int(delta * sin(w)) + 7)
        }
        else if (90 < winkel) && (winkel <= 180)
        {
            text.PositionSetzen(x: xm - Int(delta * cos(w)), y: ym + Int(delta * sin(w)))
        }
        else
        {
            text.PositionSetzen(x: xm + Int(delta * cos(w)), y: ym - Int(delta * sin(w)) + 10)
        }
    }
    
    /**
    Entfernt das Kantensymbol aus der Anzeige
    */
    func Entfernen()
    {
        außen.Entfernen()
        innen.Entfernen()
        pfeil.Entfernen()
        text.Entfernen()
    }
    
    /**
	Setzt die Farbe der Darstellung
	- parameters:
		- f: die (neue) Farbe
    */
    func FarbeSetzen(f: String)
    {
        farbe = f
        DarstellungAktualisieren()
    }
    
    /**
	Meldet den Startknoten
	- returns: Startknoten
    */
    func StartsymbolGeben() -> KnotenSymbol
    {
        return von
    }
    
    /**
	Meldet den Zielknoten
	- returns: Zielknoten
    */
    func ZielsymbolGeben() -> KnotenSymbol
    {
        return nach
    }
    
    /**
	Setzt die Sichtbarkeit des Symbols
	- parameters:
		- sichtbar: wenn wahr, ist das Symbol sichtbar
    */
    func SichtbarkeitSetzen(sichtbar: Bool)
    {
        außen.SichtbarkeitSetzen(sichtbar: sichtbar)
        innen.SichtbarkeitSetzen(sichtbar: sichtbar && (breite >= 4))
        text.SichtbarkeitSetzen(sichtbar: sichtbar)
        pfeil.SichtbarkeitSetzen(sichtbar: sichtbar && gerichtet)
    }
}

/**
Beschreibt einen Knoten

- author: Albert Wiedemann
- version: 1.0
 */
class Knoten
{
    /** x-Koordinate des Knotens */
    private var x: Int
    /** y-Koordinate des Knotens */
    private var y: Int
    /** Das Darstellungssymbol des Knotens */
    private let symbol: KnotenSymbol
    /** Vorgänger für die Wegesuche */
    private var vorgänger: Knoten?
    
    /**
	Besetzt die Attribute und legt das Knotensymbol an.
	- parameters:
		- x: x-Koordinate
		- y: y-Koordinate
    */
    init(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        symbol = KnotenSymbol(x: x * LabyrinthElement.seitenlänge + LabyrinthElement.seitenlänge / 2, y: y * LabyrinthElement.seitenlänge + LabyrinthElement.seitenlänge / 2, radius: (LabyrinthElement.seitenlänge - 4) / 2, farbe: "weiß", bezeichner: "")
    }
    
    /**
	Meldet das Symbol des Knotens zurück.
	- returns: Bezeichner
    */
    func SymbolGeben() -> KnotenSymbol
    {
        return symbol
    }
    
    /**
	Meldet den Vorgänger des Knotens zurück.
	- returns: Vorgänger
    */
    func VorgängerGeben() -> Knoten?
    {
        return vorgänger
    }
    
    /**
	Setzt den Vorgänger für den Weg zum Ausgang
	- parameters;
		- v: der Vorgänger
    */
    func VorgängerSetzen(v: Knoten)
    {
        vorgänger = v
    }
    
    /**
	Setzt die Farbe der Darstellung
	- parameters:
		- f: die (neue) Farbe
    */
    func FarbeSetzen(f: String)
    {
        symbol.FarbeSetzen(f: f)
    }
    
    /**
	Meldet die x-Koordinate des Symbols zurück.
	- returns: x-Koordinate
    */
    func XGeben() -> Int
    {
        return x
    }
    
    /**
	Meldet die y-Koordinate des Symbols zurück.
	- returns: y-Koordinate
    */
    func YGeben() -> Int
    {
        return y
    }
}


/**
Verwaltet einen ungerichteten, gewichteten Graphen mittels Adjazenzmatrix

- author: Albert Wiedemann
- version: 1.0
*/
class GraphMatrix
{
    /** Feld der Knoten des Graphen */
    private var knoten: [Knoten]
    /** 2-dim Feld der Adjazenzmatrix */
    private var matrix: [[Int]]
    /** Feld der Kantensymbole des Graphen */
    private var kanten: [KantenSymbol]
    /** Markierung für die besuchten Knoten */
    private var besuchteKnoten: [Int]
    /** Von der Tiefensuche gefundener Weg */
    private var weg: [Knoten]
    
    /**
    Baut die Datenstruktur auf
    */
    init()
    {
        knoten = [Knoten]()
        matrix = [[Int]]()
        kanten = [KantenSymbol]()
        besuchteKnoten = [Int]()
        weg = [Knoten]()
    }
    
    /**
	Einfügen eines neuen Knoten in den Graphen
	- parameters:
		- x: x-Koordinate für die Anzeige des Knotens
		- y: y-Koordinate für die Anzeige des Knotens
    */
    func KnotenEinfügen(x: Int, y: Int)
    {
        knoten.append(Knoten(x: x, y: y))
        for index in 0 ..< matrix.count
        {
            matrix[index].append(-1)
        }
        matrix.append([Int]())
        for _ in 0 ..< matrix.count
        {
            matrix[matrix.count - 1].append(-1)
        }
    }

    /**
	Gibt den Knoten mit den angegebenen Koordinaten zurück.
	
	Wenn ein Knoten mit diesen Koordinaten nicht bekannt ist, wird nil zurückgegeben
	- parameters:
		- x: x-Koordinate des Knotens
		- y: y-Koordinate des Knotens
	- returns: Referenz auf das Knotenobjekt oder nil
    */
    func KnotenGeben(x: Int, y: Int) -> Knoten?
    {
        for k in knoten
        {
            if (k.XGeben() == x) && (k.YGeben() == y)
            {
                return k
            }
        }
        return nil
    }

    /**
	Gibt die interne Nummer des Knotens
	
	Wenn ein Knoten mit diesen Koordinaten nicht bekannt ist, wird -1 zurückgegeben
	- parameters:
		- x: x-Koordinate des Knotens
		- y: y-Koordinate des Knotens
	- returns: Indexnummer des Knotens im Knotenfeld; 0 <= res <= knoten.size()-1 bzw. -1
    */
    func KnotenNummerGeben(x: Int, y: Int) -> Int
    {
        for index in 0 ..< knoten.count
        {
            let k = knoten[index]
            if (k.XGeben() == x) && (k.YGeben() == y)
            {
                return index
            }
        }
        return -1
    }

    /**
	Einfügen einer Kante in den Graphen
	
	Eine Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt und hat eine Gewichtung
	- parameters:
		- x1: x-Koordinate des Anfangsknotens
		- y1: y-Koordinate des Anfangsknotens
		- x2: x-Koordinate des Endknotens
		- y2: y-Koordinate des Endknotens
    */
    func KanteEinfügen(x1: Int, y1: Int, x2: Int, y2: Int)
    {
        let vonNummer = KnotenNummerGeben(x: x1, y: y1)
        let nachNummer = KnotenNummerGeben(x: x2, y: y2)
        if (vonNummer != -1) && (nachNummer != -1) && (vonNummer != nachNummer)
        {
            matrix[vonNummer][nachNummer] = 1
            matrix[nachNummer][vonNummer] = 1
            kanten.append(KantenSymbol(start: knoten[vonNummer].SymbolGeben(), ende: knoten[nachNummer].SymbolGeben(), gerichtet: false, gewicht: "", breite: 3, farbe: "blau"))
        }
    }

    /**
	Gibt die Anzahl der Knoten des Graphen zurück
	- returns:  Anzahl der Knoten
    */
    func KnotenAnzahlGeben() -> Int
    {
        return knoten.count
    }
    
    /**
	Gibt die Gewichtung einer Kante zurück
	
	Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt
	- parameters:
		- x1: x-Koordinate des Anfangsknotens
		- y1: y-Koordinate des Anfangsknotens
		- x2: x-Koordinate des Endknotens
		- y2: y-Koordinate des Endknotens
	- returns: Gewichtung der Kante
    */
    func KanteGewichtGeben(x1: Int, y1: Int, x2: Int, y2: Int) -> Int
    
    {
        let vonNummer = KnotenNummerGeben(x: x1, y: y1)
        let nachNummer = KnotenNummerGeben(x: x2, y: y2)
        if (vonNummer != -1) && (nachNummer != -1)
        {
            return matrix[vonNummer][nachNummer]
        }
        else
        {
            return -1
        }
    }
    
    /**
	Löscht die Kanten und Knoten des Graphen
	
	Die Anzeige wird auch gelöscht
    */
    func ZurückSetzen()
    {
        for k in kanten
        {
            k.Entfernen()
        }
        for k in knoten
        {
            k.SymbolGeben().Entfernen()
        }
        knoten.removeAll()
        kanten.removeAll()
        matrix.removeAll()
    }
    
    /**
	Setzt die Sichtbarkeit des Graphen
	- parameters:
		- sichtbar: wenn wahr, ist das Labyrinth sichtbar
    */
    func SichtbarkeitSetzen(sichtbar: Bool)
    {
        for k in kanten
        {
            k.SichtbarkeitSetzen(sichtbar: sichtbar)
        }
        for k in knoten
        {
            k.SymbolGeben().SichtbarkeitSetzen(sichtbar: sichtbar)
        }
    }
   
    /**
	Führt die Tiefensuche vom Startknoten aus durch
	- parameters:
		- x1:  x-Koordinate des Startknotens
		- y1: y-Koordinate des Startknotens
		- x2: x-Koordinate des Zielknotens
		- y2: y-Koordinate des Zielknotens
	- returns: Weg als Feld von Knoten
    */
    func TiefensucheAusführen (x1: Int, y1: Int, x2: Int, y2: Int) -> [Knoten]
    {
        besuchteKnoten.removeAll()
        weg.removeAll()
        KnotenBesuchen(aktuell: KnotenNummerGeben(x: x1, y: y1), ziel: KnotenNummerGeben(x: x2, y: y2))
        return weg
    }
    
    /**
	Setzt den Rekursionsschritt bei einem Knoten um.
	- parameters:
		- aktuell: die Nummer des aktuell zu besuchenden Knotens
		- ziel: die Nummer des Zielknotens
    */
    func KnotenBesuchen(aktuell: Int, ziel: Int) -> Bool
    {
        besuchteKnoten.append(aktuell)
        weg.append(knoten[aktuell])
        if aktuell == ziel
        {
            return true
        }
        else
        {
            for nummer in 0 ..< knoten.count
            {
                if (matrix[aktuell][nummer] > 0) && (!besuchteKnoten.contains(nummer))
                {
                    if KnotenBesuchen(aktuell: nummer, ziel: ziel)
                    {
                        return true
                    }
                }
            }
            weg.remove(at: weg.count - 1)
            return false
        }
    }
}

/**
Rahmenklasse für die Wegesuche aus dem Labyrinth.

- author: Albert Wiedemann
- version: 1.0
 */
class Rahmen
{
    /** Das Labyrinth */
    private let l: Labyrinth
    /** Der zugehörige Graph */
    private let g: GraphMatrix
    /** Der verirrte Theseus */
    private var t: Theseus?

    /**
    Legt die benötigten Objekte an.
    */
    init()
    {
        l = Labyrinth(breite: 20, höhe: 13)
        l.Generieren()
        g = GraphMatrix()
        l.GraphErzeugen(graph: g)
        t = nil
    }
    
    /**
	Zeigt nur den Graphen an.
    */
    func GraphAnzeigen()
    {
        l.SichtbarkeitSetzen(sichtbar: false)
        g.SichtbarkeitSetzen(sichtbar: true)
        if t != nil
        {
            t!.SichtbarkeitSetzen(sichtbar: false)
        }
    }
    
    /**
	Zeigt nur das Labyrinth an.
    */
    func LabyrinthAnzeigen()
    {
        l.SichtbarkeitSetzen(sichtbar: true)
        g.SichtbarkeitSetzen(sichtbar: false)
        if t != nil
        {
            t!.SichtbarkeitSetzen(sichtbar: true)
        }
    }
    
    /**
	Positioniert den verirrten Theseus auf einen geeigneten Startpunkt
     */
    private func TheseusPositionieren()
    {
        if t != nil
        {
            t!.Entfernen()
        }
        LabyrinthAnzeigen()
        l.StartpunktGenerieren()
        t = Theseus(x: l.XStartGeben(), y: l.YStartGeben())
    }
    
    /**
	Ermittelt den Weg aus dem Labyrinth und bereitet den Gang zu Ausgang vor.
    */
    private func FluchtwegBerechnen()
    {
        if t != nil
        {
            t!.WegSetzen(weg: g.TiefensucheAusführen(x1: l.XStartGeben(), y1: l.YStartGeben(), x2: l.XAusgangGeben(), y2: l.YAusgangGeben()));
        }
    }
    
    /**
	Setzt Theseus auf eine Startposition und setzt den Weg zum Ausgang
    */
    func FluchtAusführen()
    {
        TheseusPositionieren()
        FluchtwegBerechnen()
    }
}

let r = Rahmen()
//r.GraphAnzeigen()
r.LabyrinthAnzeigen()
r.FluchtAusführen()
